"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const generators_1 = require("../../generators");
const lodash_1 = __importDefault(require("lodash"));
// These are just example conditional closures
const closures = [
    /**
     * always
     * Always true
     */
    generators_1.closureGenerator('always', function (_facts, _context) {
        return true;
    }, {}),
    /**
     * equal - value1 === value2
     * @param context.parameters.value1
     * @param context.parameters.value2
     * @return boolean
     **/
    generators_1.closureGenerator('equal', (_facts, context) => {
        var _a, _b;
        return ((_a = context.parameters) === null || _a === void 0 ? void 0 : _a.value1) === ((_b = context.parameters) === null || _b === void 0 ? void 0 : _b.value2);
    }, { required: ['value1', 'value2'] }),
    /**
     * not-equal - value1 !== value2
     * @param context.parameters.value1
     * @param context.parameters.value2
     * @return boolean
     **/
    generators_1.closureGenerator('not-equal', (_facts, context) => {
        var _a, _b;
        return ((_a = context.parameters) === null || _a === void 0 ? void 0 : _a.value1) !== ((_b = context.parameters) === null || _b === void 0 ? void 0 : _b.value2);
    }, { required: ['value1', 'value2'] }),
    /**
     * contains
     *
     * example: {closure: 'contains', value: "someLongString", contains: 'Long'} - Returns true
     * @param context.parameters.value    - The longer value that will contain another value
     * @param context.parameters.contains - The smaller value that will be contained in "value"
     * @return boolean - True if "contains" is contained in value
     **/
    generators_1.closureGenerator('contains', (_facts, context) => {
        var _a, _b;
        return (((_a = context.parameters) === null || _a === void 0 ? void 0 : _a.value) || '').includes((_b = context.parameters) === null || _b === void 0 ? void 0 : _b.contains);
    }, { required: ['value', 'contains'] }),
    /**
     * allFieldsDefined - Returns true if all parameter fields are defined (not undefined)
     * example {closure: 'allFieldsDefined', fields: ['something.name']} - If something.name is defined (not undefined)
     * @param any number of params
     * @return boolean
     **/
    generators_1.closureGenerator('allFieldsDefined', (facts, context) => {
        let ret = true;
        for (let field of context.parameters.fields) {
            ret = lodash_1.default.get(facts, field) !== undefined;
            if (!ret)
                break;
        }
        return ret;
    }, { required: ['fields'] }),
    /**
     * allFieldsUndefined - Returns true if all parameter fields are undefined
     * example {closure: 'allFieldsUndefined', fields: ['something.name']} - If something.name is undefined
     * @param any number of params
     * @return boolean
     **/
    generators_1.closureGenerator('allFieldsUndefined', (facts, context) => {
        let ret = true;
        for (let field of context.parameters.fields) {
            ret = lodash_1.default.get(facts, field) === undefined;
            if (!ret)
                break;
        }
        return ret;
    }, { required: ['fields'] }),
    /**
     * not - Revert the closure return value
     * example {closure: 'not' notClosure: 'allFieldsUndefined', fields: ['something.name']} - If something.name is defined
     * @param any number of params
     * @return boolean - true if all field paths match
     **/
    generators_1.closureGenerator('not', (facts, context) => __awaiter(void 0, void 0, void 0, function* () {
        return !(yield context.parameters.notClosure.process(facts, context));
    }), { required: ['notClosure'], closureParameters: ['notClosure'] }),
];
exports.default = closures;
//# sourceMappingURL=conditionals.js.map