"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const generators_1 = require("../../generators");
const lodash_1 = __importDefault(require("lodash"));
const closures = [
    /**
     * concat
     * set facts with some extra values.
     *  example: { closure: 'concat', key: 'some.path', value: 1 } to set facts.some.path = 1
     *  example: { closure: 'concat', key: 'some.path', '^value': 'some.other.path' } to set facts.some.path = facts.some.other.path
     */
    generators_1.closureGenerator('concat', function (facts, context) {
        let key = context.parameters.key;
        if (key) {
            let keyValue = lodash_1.default.get(facts, key);
            let value = context.parameters.value;
            lodash_1.default.set(facts, key, keyValue + value);
        }
        return facts;
    }, {}),
    /**
     * set
     * set facts with some extra values.
     *  example: { closure: 'set', key: 'some.path', value: 1 } to set facts.some.path = 1
     *  example: { closure: 'set', key: 'some.path', '^value': 'some.other.path' } to set facts.some.path = facts.some.other.path
     */
    generators_1.closureGenerator('set', function (facts, context) {
        let key = context.parameters.key;
        let value = context.parameters.value;
        if (key)
            lodash_1.default.set(facts, key, value);
        return facts;
    }, {}),
    /**
     * Push
     * Pushes a value onto an array. It is possible to push onto multiple arrays by providing multiple parameters
     *  example: { closure: 'push', key: 'path.to.some.array', value: 1 } - Facts will = {path: {to: { some: {array: [1]}}}} or it may have other values on the array if they were previosuly push to the array
     *  example: { closure: 'push', key: 'path.to.some.array', '^value': 'path.to.some.value' } - The value that will be pushed onto the array will be from 'path.to.some.value'
     */
    generators_1.closureGenerator('push', function (facts, context) {
        let array = lodash_1.default.get(facts, context.parameters.key);
        let value = context.parameters.value;
        if (!Array.isArray(array)) {
            array = [];
        }
        array.push(value);
        lodash_1.default.set(facts, context.parameters.key, array);
        return facts;
    }, {}),
];
exports.default = closures;
//# sourceMappingURL=transformations.js.map