/**
 * Core AMQP Message - Use for the Core AMQP Input
 *
 * This object is used by AMQP Input to pass in all the pieces of a message that has been received.
 *
 * Refer to https://www.squaremobius.net/amqp.node/channel_api.html#channel_consume for details
 * on message fields and properties.
 */
export interface ICoreAmqpMessage {
    amqpMessageContent: string;
    amqpMessageFields: object;
    amqpMessageProperties: object;
}
/**
 * Core AMQP Publish Action - Used for the Core AMQP Output
 *
 * This object is used to control what the Core AMQP Output publishes.
 *
 * This is intended to be passed inside an ARRAY of these in order to send multiple messages!
 */
export interface ICoreAmqpPublishAction {
    amqpPublishExchange: string;
    amqpMessageContent: string;
    amqpPublishRoutingKey?: string;
    amqpPublishOptions?: object;
}
/**
 * Core AMQP RPC Publish Action - Used for the Core AMQP RPC Output
 *
 * This object is used to control what the Core AMQP RPC Output publishes.
 *
 * This is intended to be passed as a single property inside an updated facts object amqpRpcPublishAction.
 */
export interface ICoreAmqpRpcPublishAction {
    amqpRpcResponse: string;
}
